package demo.android.life;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.TextView;

public class RunActivity extends Activity implements OnClickListener
{
    public static final String EXT_COLS = "cols";
    public static final String EXT_ROWS = "rows";
    public static final String EXT_CELLS = "cells";
    
    private GridView mLifeGrid;
    private LifeAdapter mAdapter;
    Button mCloseButton;
    
    private Handler mHandler;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.run);
        
        mCloseButton = (Button) findViewById(R.id.close);
        mCloseButton.setOnClickListener(this);

        Bundle extras = getIntent().getExtras();
        int cols = extras.getInt(EXT_COLS);
        int rows = extras.getInt(EXT_ROWS);
        int cells = extras.getInt(EXT_CELLS);
        mAdapter = new LifeAdapter(this, cols, rows, cells);
        
        mLifeGrid = (GridView)findViewById(R.id.life_grid);
        mLifeGrid.setAdapter(mAdapter);
        mLifeGrid.setNumColumns(cols);
        mLifeGrid.setEnabled(false);
        mLifeGrid.setStretchMode(0);

        mHandler = new Handler();
        mHandler.postDelayed(mUpdateGeneration, 1000);
    }

    private Runnable mUpdateGeneration = new Runnable()
    {
        public void run()
        {
            mAdapter.next();
            mLifeGrid.setAdapter(mAdapter);

            mHandler.postDelayed(mUpdateGeneration, 1000);
        }
    }; 


    public void onClick(View view)
    {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        
        builder
            .setMessage(R.string.submit_close)
            //  "Yes",      
            .setPositiveButton(R.string.yes, new DialogInterface.OnClickListener()
                {
                    public void onClick(DialogInterface dialog, int whichButton) 
                    {
                        finish();                           
                    }
                })
             //  "No",       
            .setNegativeButton(R.string.no, new DialogInterface.OnClickListener()
                {
                    public void onClick(DialogInterface dialog, int whichButton) 
                    {
                        
                    }
                })
            .show();
    }
    
    class LifeAdapter extends BaseAdapter
    {
        private Context mContext;

        private LifeModel mLifeModel;

        public LifeAdapter(Context context, int cols, int rows, int cells)
        {
            mContext = context;
            mLifeModel = new LifeModel(rows, cols, cells);
        }

        public void next()
        {
            mLifeModel.next();
        }

        /**
         *     GridView 
         */
        public int getCount()
        {
            return mLifeModel.getCount();
        }

        /**
         *  ,    position
         */
        public Object getItem(int position)
        {
            return mLifeModel.isCellAlive(position);
        }

        /**
         *   ,     position
         */
        public long getItemId(int position)
        {
            return position;
        }

        /**
         *   ,      position
         */
        public View getView(int position, View convertView, ViewGroup parent)
        {
            ImageView view; //     
            
            if (convertView == null)
            {
                view = new ImageView(mContext);

                //  
                view.setLayoutParams(new GridView.LayoutParams(10, 10));
                view.setAdjustViewBounds(false);
                view.setScaleType(ImageView.ScaleType.CENTER_CROP);
                view.setPadding(1, 1, 1, 1);
            }
            else
            {
                view = (ImageView)convertView;
            }
            
            //   ,   ,  ,   
            view.setImageResource(mLifeModel.isCellAlive(position) ? R.drawable.cell : R.drawable.empty);
            
            return view;
        }
    }
}
