package demo.android.life;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;

public class StartActivity extends Activity implements OnClickListener
{
    //   
    private static final int ALERT_NONE = 0; //   
    private static final int ALERT_COLUMNS = 1; //   
    private static final int ALERT_ROWS = 2; //   
    private static final int ALERT_CELLS = 3; //    

    //     
    private static final int COLUMNS_MIN = 5;
    private static final int COLUMNS_MAX = 25;

    //     
    private static final int ROWS_MIN = 5;
    private static final int ROWS_MAX = 35;

    Button mButton;
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mButton = (Button) findViewById(R.id.run);
        mButton.setOnClickListener(this);
    }
    
    public void onClick(View v)
    {
        EditText rowsEditor = (EditText)findViewById(R.id.rows_count);
        EditText colsEditor = (EditText)findViewById(R.id.columns_count);
        EditText cellsEditor = (EditText)findViewById(R.id.cells_count);
        
        int cols = Integer.parseInt(colsEditor.getText().toString());
        int rows = Integer.parseInt(rowsEditor.getText().toString());
        int cells = Integer.parseInt(cellsEditor.getText().toString());
        int alertCode = checkInputParameters(cols, rows, cells);
        if (alertCode != ALERT_NONE)
        {
            showDialog(alertCode);
            return;
        }
        Intent intent = new Intent();
        intent.setClass(this, RunActivity.class);
        
        intent.putExtra(RunActivity.EXT_COLS, cols);
        intent.putExtra(RunActivity.EXT_ROWS, rows);
        intent.putExtra(RunActivity.EXT_CELLS, cells);

        startActivity(intent);
        finish();
    }
    
    private int checkInputParameters(int cols, int rows, int cells)
    {
        if (cols < COLUMNS_MIN || cols > COLUMNS_MAX)
        {
            return ALERT_COLUMNS;
        }
        
        if (rows < ROWS_MIN || rows > ROWS_MAX)
        {
            return ALERT_ROWS;
        }
        
        if (cells > rows * cols)
        {
            return ALERT_CELLS;
        }
        
        return ALERT_NONE;
    }

    @Override
    protected Dialog onCreateDialog(int id)
    {
        DialogInterface.OnClickListener doNothing = new DialogInterface.OnClickListener()
        {
            public void onClick(DialogInterface dialog, int whichButton) 
            {
            }
        };
        int alertMessage;
        
        switch (id)
        {
            case ALERT_COLUMNS:
                alertMessage = R.string.alert_columns;
                break;
            case ALERT_ROWS:
                alertMessage = R.string.alert_rows;
                break;
            case ALERT_CELLS:
                alertMessage = R.string.alert_cells;
                break;
            default:
                return null;
        }
        
        return new AlertDialog.Builder(this)
            .setMessage(alertMessage)
            .setNeutralButton(R.string.ok, doNothing)
            .create(); 
    }

}
